//
//  Path
//
//  Created by Shawn on 2007-11-06.
//

/*
The Path class is used to create and maintain PathSegments.
DrawPath uses a Path instance to draw the user interface,
as well as to update the PathSegments with changes as the user
interacts with the interface.

The main "pathList" object is a doubly-linked list, in which each element 
is a helix (or virtual strand).  In turn, each helix is a doubly-linked list
of PathSegments, which store 21-base sections of the DNA structure.

Whenever the boundaries of a structure are extended using the slice interface,
a call is made to extend the segments in each helix.

*/

package edu.harvard.med.nano
{
    import edu.harvard.med.nano.DrawPath;
    import edu.harvard.med.nano.PathSegment;
    import de.polygonal.ds.*;
    
    public class Path {
        public var drawPath:DrawPath;
        public var leftSliceIndex:int;
        public var rightSliceIndex:int;
        public var pathList:DLinkedList;
        public var pathIter:DListIterator;
        
        function Path() {
            this.pathList = new DLinkedList();
            this.pathIter = this.pathList.getListIterator();
        }
        
        public function addHelix(helixNum:int):void {
            var segList:DLinkedList = new DLinkedList();
            // loop through all slices and add a segment for each
            
            var ps:PathSegment;
            for (var i:int = leftSliceIndex; i <= rightSliceIndex; i++) {
                ps = new PathSegment(helixNum, i)
                segList.append(ps);
                this.drawPath.addDrawSeg(ps.drawSegment);
                ps.drawSegment.pathSegment = ps;
            }
            this.pathList.append(segList);
            this.drawPath.update();
        }
        
        public function extendHelix(helixNum:int):void {
        }
        
        public function prependSegment(segIndex:int):void {
            // loop through paths, add left-end segment
            var helixNum:int;
            var segList:DLinkedList;
            var iter:DListIterator = this.pathList.getListIterator();
            for (iter.start(); iter.valid(); iter.forth())
            {
                // segList is the list of segments within this strand
                segList = iter.node.data;
                // grab the helix number from the end
                helixNum = segList.head.data.helixNum;
                iter.data.prepend(new PathSegment(helixNum, segIndex));
            }
        }
        
        public function appendSegment(segIndex:int):void {
            // loop through paths, add right-end segment
            var helixNum:int;
            var segList:DLinkedList;
            var iter:DListIterator = this.pathList.getListIterator();
            
            // iter gives each virtual strand
            for (iter.start(); iter.valid(); iter.forth())
            {
                // segList is the list of segments within this strand
                segList = iter.node.data;
                // grab the helix number from the end
                helixNum = segList.head.data.helixNum;
                iter.data.append(new PathSegment(helixNum, segIndex));
            }
        }
        
        
        // When SliceNodes are updated by clicking in the slice panel
        // it is necessary to link the corresponding PathSegments that are
        // created so stage coordinates can be easily determined when
        // crossovers are drawn.
        public function linkPathSegments(helix1:int, helix2:int, p:int):void {
            var helixNum:int;
            var pathIter:DListIterator = this.pathList.getListIterator();
            
            var segIter1,segIter2:DListIterator;
            var iter2:DListIterator = this.pathList.getListIterator();
            var ps1, ps2:PathSegment;
            
            // Step 1: find target helices in pathList
            for (pathIter.start(); pathIter.valid(); pathIter.forth())
            {
                if (helix1 == pathIter.node.data.head.data.helixNum) {
                    segIter1 = pathIter.node.data.getListIterator();
                } else if (helix2 == pathIter.node.data.head.data.helixNum) {
                    segIter2 = pathIter.node.data.getListIterator();
                }
            }
            
            // Step 2: walk down each segList and link corresponding PathSegments
            var i,j:int;
            for (i=leftSliceIndex; i<=rightSliceIndex; i++) {
                ps1 = segIter1.node.data;
                ps2 = segIter2.node.data;
                segIter1.forth();
                segIter2.forth();
                switch (p) {
                    case 0:
                        ps1.p0 = ps2;
                        ps2.p0 = ps1;
                        break;
                    case 1:
                        ps1.p1 = ps2;
                        ps2.p1 = ps1;
                        break;
                    case 2:
                        ps1.p2 = ps2;
                        ps2.p2 = ps1;
                        break;
                }
            }
        }
        
        // updates Path when slice helices are added
        public function pairHelices(segIndex:int, even:int, odd:int, p:int):void {
            var pos:int = getScafXoverPos(p);
            var helixNum:int;
            var segList:DLinkedList;
            var iter:DListIterator = this.pathList.getListIterator(); // iter gives each virtual strand
            
            // loop through paths, update PathSegment crossovers
            for (iter.start(); iter.valid(); iter.forth())
            {
                // segList is the list of segments within this strand
                segList = iter.node.data;
                // grab the helix number from the end
                helixNum = segList.head.data.helixNum;
                var segIter:DListIterator;
                // Find left-hand PathSegment for odd helix, add xover
                if (helixNum == even) {
                    segIter = segList.getListIterator();
                    segIter.start();
                    // print position
                    for (var i:int=leftSliceIndex; i<segIndex; i++) {
                        segIter.forth();
                    }
                    segIter.node.data.addScafXover(pos,odd);
                }
                // Advance to right-hand PathSegment for even helix, add xover
                if (helixNum == odd) {
                    segIter = segList.getListIterator();
                    segIter.start();
                    // print position
                    for (var j:int=leftSliceIndex; j<segIndex-1; j++) {
                        segIter.forth();
                    }
                    segIter.node.data.addScafXover(pos,even);
                }
            }
        }
        
        // return scaffold crossover position (default to leftmost position)
        private function getScafXoverPos(p:int, left:Boolean=true):int {
            var pos:int;
            // all positions -1 for zero-based indexing
            if (left) {  // left: 5, 9, 2
                if (p == 0) {
                    pos = 4;
                } else if (p == 1) {
                    pos = 8;
                } else if (p == 2) {
                    pos = 1;
                }
            } else {    // right: 16, 19, 12
                if (p == 0) {
                    pos = 15;
                } else if (p == 1) {
                    pos = 18;
                } else if (p == 2) {
                    pos = 11;
                }
            }
            return pos;
        }
    }
}
